from pymfl.api.MFLAPIClient import MFLAPIClient


class FantasyContentAPIClient(MFLAPIClient):

    @classmethod
    def get_players(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        TODO: caching for this method
        All player IDs, names and positions that MyFantasyLeague.com has in our database for the current year.
        All other data types refer only to player IDs, so if you'd like to later present any data to people,
        you'll need this data type for translating player IDs to player names.
        Our player database is updated at most once per day, and it contains more than 2,000 players.
        In other words, you're strongly encouraged to read this data type no more than once per day and store it locally as needed to optimize your system performance.
        """
        filters = [("TYPE", "players"), ("JSON", 1)]
        # Set this value to 1 to return complete player details, including player IDs from other sources.
        details: int = kwargs.pop("details", None)
        # Pass a unix timestamp via this parameter to receive only changes to the player database since that time.
        since: str = kwargs.pop("since", None)
        # Pass a list of player ids separated by commas (or just a single player id) to receive back just the info on those players.
        players: str = kwargs.pop("players", None)
        cls._add_filter_if_given("DETAILS", details, filters)
        cls._add_filter_if_given("SINCE", since, filters)
        cls._add_filter_if_given("PLAYERS", players, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_player_profile(cls, *, year: int, league_id: str, player_id_or_ids: str) -> dict:
        """
        Returns a summary of information regarding a player, including DOB, ADP ranking, height/weight.

        player_id_or_ids: Player id or list of player ids separated by commas
        """
        filters = [("TYPE", "playerProfile"), ("P", player_id_or_ids), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_all_rules(cls, *, year: int, league_id: str) -> dict:
        """
        All scoring rules that MyFantasyLeague.com currently supports, including:
        if the rule is scored for players, teams or coaches, as well as an abbreviation of the scoring rule, a short description, and a detailed description.
        If you plan on using the 'rules' data type, you'll also need this data type to look up the abbreviations to translate them to their detailed description for people.
        """
        filters = [("TYPE", "allRules"), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_player_ranks(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        This report provides overall player rankings from the experts at FantasySharks.com.
        These rankings can be used instead of Average Draft Position (ADP) rankings for guidance during your draft, or when generating your own draft list.
        """
        filters = [("TYPE", "playerRanks"), ("JSON", 1)]
        position: str = kwargs.pop("POS", None)
        cls._add_filter_if_given("POS", position, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_adp(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        ADP results, including:
            - When the result were last updated
            - How many drafts the player was selected in
            - The average pick
            - Minimum pick
            - Maximum pick
        """
        filters = [("TYPE", "adp"), ("JSON", 1)]
        # This returns draft data for just drafts that started after the specified period. Valid values are ALL, RECENT, DRAFT, JUNE, JULY, AUG1, AUG15, START, MID, PLAYOFF.
        # This option is not valid for previous seasons.
        period: str = kwargs.pop("PERIOD", None)
        # This returns draft data from just leagues with this number of franchises.
        # Valid values are 8, 10, 12, 14 or 16.
        # If the value is 8, it returns data from leagues with 8 or fewer franchises.
        # If the value is 16 it returns data from leagues with 16 or more franchises.
        f_count: int = kwargs.pop("f_count", None)
        # Filters the data returned as follows:
        #   - If set to 0, data is from leagues that not use a PPR scoring system
        #   - If set to 1, only from PPR scoring system
        #   - If set to -1 (or not set), all leagues
        is_ppr: int = kwargs.pop("is_ppr", None)
        # Pass a string with some combination of N, K and R:
        #   - If N specified, returns data from redraft leagues
        #   - If 'K' is specified, returns data for keeper leagues
        #   - If 'R' is specified, return data from rookie-only drafts.
        # You can combine these.
        # If you specify 'KR' it will return rookie and keeper drafts only. Default is 'NKR'.
        is_keeper: str = kwargs.pop("is_keeper", None)
        # If set to 1, returns data from mock draft leagues only.
        # If set to 0, excludes data from mock draft leagues.
        # If set to -1, returns all
        is_mock: int = kwargs.pop("is_mock", None)
        # Only returns data for players selected in at least this percentage of drafts.
        # So if you pass 10, it means that players selected in less than 10% of all drafts will not be returned.
        # Note that if the value is less than 5, the results may be unpredictable.
        cutoff: int = kwargs.pop("cutoff", None)
        # If set to 1, it returns the leagues that were included in the results.
        # This option only works for the current season.
        details: int = kwargs.pop("details", None)

        cls._add_filter_if_given("PERIOD", period, filters)
        cls._add_filter_if_given("FCOUNT", f_count, filters)
        cls._add_filter_if_given("IS_PPR", is_ppr, filters)
        cls._add_filter_if_given("IS_KEEPER", is_keeper, filters)
        cls._add_filter_if_given("IS_MOCK", is_mock, filters)
        cls._add_filter_if_given("CUTOFF", cutoff, filters)
        cls._add_filter_if_given("DETAILS", details, filters)

        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_aav(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        AAV results, including:
            - When the result were last updated
            - How many auctions the player was selected in
            - Average auction value
        Average auction value is relative to an auction where a total of $1000 is available across all franchises.
        """
        filters = [("TYPE", "aav"), ("JSON", 1)]
        # This returns auction data for just auctions that started after the specified period.
        # Valid values are ALL, RECENT, DRAFT, JUNE, JULY, AUG1, AUG15, START, MID, PLAYOFF.
        # This option is not valid for previous seasons.
        period: str = kwargs.pop("period", None)
        # Filters the data returned as follows:
        #   - If set to 0, data is from leagues that not use a PPR scoring system
        #   - If set to 1, only from PPR scoring system
        #   - If set to -1 (or not set), all leagues
        is_ppr: int = kwargs.pop("is_ppr", None)
        # Pass a string with some combination of N, K and R:
        #   - If N specified, returns data from redraft leagues
        #   - If 'K' is specified, returns data for keeper leagues
        #   - If 'R' is specified, return data from rookie-only drafts
        # You can combine these.
        # If you specify 'KR' it will return rookie and keeper drafts only.
        # Default is 'NKR'.
        is_keeper: str = kwargs.pop("is_keeper", None)

        cls._add_filter_if_given("PERIOD", period, filters)
        cls._add_filter_if_given("IS_PPR", is_ppr, filters)
        cls._add_filter_if_given("IS_KEEPER", is_keeper, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_top_adds(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The most added players across all MyFantasyLeague.com-hosted leagues during the current week
        (or the past 7 days during the pre-season), as well as the percentage of leagues that they've been added in.
        Only players that have been added in at least 1% of our leagues will be returned.
        This data would be helpful in creating some sort of "Who's Hot?" list.
        """
        filters = [("TYPE", "topAdds"), ("JSON", 1)]
        # Limits the result to this many players.
        count: int = kwargs.pop("count", None)
        # Set this value to FA to only return available free agents.
        # This option is not available for deluxe leagues.
        status: str = kwargs.pop("status", None)
        cls._add_filter_if_given("COUNT", count, filters)
        cls._add_filter_if_given("STATUS", status, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_top_drops(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The most dropped players across all MyFantasyLeague.com-hosted leagues during the current week
        (or the past 7 days during the pre-season), as well as the percentage of leagues that they've been dropped in.
        Only players that have been dropped in at least 1% of our leagues will be displayed.
        This data would be helpful in creating some sort of "Who's Cold?" list.
        """
        filters = [("TYPE", "topDrops"), ("JSON", 1)]
        # Limits the result to this many players.
        count: int = kwargs.pop("count", None)
        # Set this value to FA to only return available free agents.
        # This option is not available for deluxe leagues.
        status: str = kwargs.pop("status", None)
        cls._add_filter_if_given("COUNT", count, filters)
        cls._add_filter_if_given("STATUS", status, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_top_starters(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The most started players across all MyFantasyLeague.com-hosted leagues for the current week,
        as well as the percentage of leagues that they've been started in.
        Only players that have been started in at least 1% of our leagues will be displayed.
        """
        filters = [("TYPE", "topStarters"), ("JSON", 1)]
        # Limits the result to this many players.
        count: int = kwargs.pop("count", None)
        # Set this value to FA to only return available free agents.
        # This option is not available for deluxe leagues.
        status: str = kwargs.pop("status", None)
        cls._add_filter_if_given("COUNT", count, filters)
        cls._add_filter_if_given("STATUS", status, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_top_trades(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The most traded players across all MyFantasyLeague.com-hosted leagues in the current week
        (or past 7 days during the pre-season),
        as well as the percentage of leagues that they have been traded in.
        Only players that are traded in more than 0.25% of our leagues will be displayed.
        """
        filters = [("TYPE", "topTrades"), ("JSON", 1)]
        # Limits the result to this many players.
        count: int = kwargs.pop("count", None)
        cls._add_filter_if_given("COUNT", count, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_top_owns(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The most owned players across all MyFantasyLeague.com-hosted leagues,
        as well as the percentage of leagues that they're owned in.
        Only players that are owned in more than 2% of our leagues will be displayed.
        """
        filters = [("TYPE", "topOwns"), ("JSON", 1)]
        # Limits the result to this many players.
        count: int = kwargs.pop("count", None)
        # Set this value to FA to only return available free agents.
        # This option is not available for deluxe leagues.
        status: str = kwargs.pop("status", None)
        cls._add_filter_if_given("COUNT", count, filters)
        cls._add_filter_if_given("STATUS", status, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_site_news(cls, *, year: int, league_id: str) -> dict:
        """
        An RSS feed of MyFantasyLeague.com site news.
        """
        filters = [("TYPE", "siteNews"), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_who_should_i_start(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        Site-wide 'Who Should I Start?' data - offering a comparison between any two players at the same position,
        letting you know what percent of all MFL customers would choose one player over another player.
        Access restricted to league owners.
        """
        filters = [("TYPE", "whoShouldIStart"), ("JSON", 1)]
        # If specified, filter the results to the passed player ids (separated by commas).
        players: str = kwargs.pop("players", None)
        cls._add_filter_if_given("PLAYERS", players, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)



from pymfl.api.MFLAPIClient import MFLAPIClient
from pymfl.util import ConfigReader


class SessionAPIClient(MFLAPIClient):
    __LOGIN_ROUTE = ConfigReader.get("api", "login_route")

    @classmethod
    def get_mfl_user_id(cls, *, year: int, username: str, password: str) -> str:
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls.__LOGIN_ROUTE)
        url = cls._add_filters(url, ("USERNAME", username), ("PASSWORD", password), ("XML", 1))
        response = cls._post(url, as_xml=True)
        return response.attrib["MFL_USER_ID"]  # TODO: do some error handling here



from pymfl.api.MFLAPIClient import MFLAPIClient
from pymfl.enum import APIResponseType


class OtherLeagueInfoAPIClient(MFLAPIClient):

    @classmethod
    def get_future_draft_picks(cls, *, year: int, league_id: str) -> dict:
        """
        Future draft picks for a given league.
        Private league access restricted to league owners.
        """
        filters = [("TYPE", "futureDraftPicks"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_accounting(cls, *, year: int, league_id: str) -> dict:
        """
        Returns a summary of the league accounting records.
        In the response, negative amounts are charges against the franchise while positive amounts is money paid by the franchise or owed to the franchise.
        """
        filters = [("TYPE", "accounting"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_pool(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        Returns a summary of the league accounting records.
        In the response, negative amounts are charges against the franchise while positive amounts is money paid by the franchise or owed to the franchise.
        """
        filters = [("TYPE", "pool"), ("L", league_id), ("JSON", 1)]
        # Which pool picks to return.
        # Valid values are "NFL" (default) or "Fantasy".
        pool_type: str = kwargs.pop("pool_type", None)
        cls._add_filter_if_given("POOLTYPE", pool_type, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_survivor_pool(cls, *, year: int, league_id: str) -> dict:
        """
        All survivor pool picks for a given league.
        Private league access restricted to league owners.
        """
        filters = [("TYPE", "survivorPool"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_abilities(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        Returns the abilities of the current franchise.
        A value of 0 means the franchise does not have the ability and a value of 1 means it has the ability. Test it!
        Access restricted to league owners.
        """
        filters = [("TYPE", "abilities"), ("L", league_id), ("JSON", 1)]
        # Franchise ID.
        # When the request comes from the commissioner, this indicates which franchise's abilities to return.
        # It's ignored if the request comes from an owner.
        franchise_id: str = kwargs.pop("franchise_id", None)
        details: int = kwargs.pop("details", None)
        cls._add_filter_if_given("F", franchise_id, filters)
        cls._add_filter_if_given("DETAILS", details, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_appearance(cls, *, year: int, league_id: str) -> dict:
        """
        The skin, home page tabs, and modules within each tab set up by the commissioner for a given league.
        """
        filters = [("TYPE", "appearance"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_rss(cls, *, year: int, league_id: str) -> dict:
        """
        An RSS feed of key league data for a given league, including:
            - league standings
            - current week's live scoring
            - last week's fantasy results
            - the five newest message board topics
        """
        filters = [("TYPE", "rss"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_ics(cls, *, year: int, league_id: str) -> str:
        """
        Returns a summary of the league calendar in .ics format.
        This format is suitable for importing into many modern calendaring programs such as:
            - Apple's Calendar
            - Google Calendar
            - Microsoft Outlook
        Access restricted to league owners.
        """
        filters = [("TYPE", "ics"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        response = cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id,
                                                   api_response_type=APIResponseType.CONTENT)
        return response.decode("utf-8")



import xml.etree.ElementTree as ET
from abc import ABC
from http import HTTPStatus
from typing import Optional, Any

import requests
from requests import Response

from pymfl.api.config import APIConfig
from pymfl.enum import APIResponseType
from pymfl.util import ConfigReader


class MFLAPIClient(ABC):
    """
    Should be inherited by all API Clients.
    Sleeper API Documentation: https://api.myfantasyleague.com/2022/api_info
    """
    __API_CONFIG = APIConfig
    _MFL_APP_BASE_URL = ConfigReader.get("api", "mfl_app_base_url")

    # ROUTES
    _EXPORT_ROUTE = ConfigReader.get("api", "export_route")

    @classmethod
    def _build_route(cls, base_url: str, *args) -> str:
        args = (str(arg).replace("/", "") for arg in args)
        return f"{base_url}/{'/'.join(args)}"

    @classmethod
    def _add_filters(cls, url: str, *args) -> str:
        """
        Adds filters to the given url.
        """
        if len(args) > 0:
            symbol = "?"
            for i, arg in enumerate(args):
                if i > 0:
                    symbol = "&"
                url = f"{url}{symbol}{arg[0]}={arg[1]}"
        return url

    @staticmethod
    def _add_filter_if_given(key: str, value: Optional[Any], filter_list: list[tuple[str, Any]]):
        """
        Helper method for adding filters.
        """
        if value is not None:
            filter_list.append((key, value))

    @classmethod
    def _get_for_year_and_league_id(cls, *, url: str, year: int, league_id: str,
                                    api_response_type: APIResponseType = APIResponseType.JSON) -> dict | bytes:
        response = cls.__get_response_for_year_and_league_id(url=url, year=year, league_id=league_id)
        if api_response_type == APIResponseType.JSON:
            return response.json()
        elif api_response_type == APIResponseType.CONTENT:
            return response.content

    @classmethod
    def __get_response_for_year_and_league_id(cls, *, url: str, year: int, league_id: str) -> Response:
        api_config = cls.__API_CONFIG.get_config_by_year_and_league_id(year=year, league_id=league_id)
        cookies = {"MFL_LAST_LEAGUE_ID": api_config.league_id, "MFL_USER_ID": api_config.mfl_user_id}
        response = requests.get(url, cookies=cookies)
        if response.status_code != HTTPStatus.OK:
            raise Exception("BAD STATUS CODE")  # TODO: better error handling
        return response

    @staticmethod
    def _post(url: str, body: dict = None, **kwargs) -> dict | ET.Element:
        if body is None:
            body = dict()
        as_xml = kwargs.pop("as_xml")
        response = requests.post(url, data=body)
        if response.status_code != HTTPStatus.OK:
            raise Exception("BAD STATUS CODE")  # TODO: better error handling
        if as_xml:
            return ET.fromstring(response.content)
        return response.json()



from pymfl.api.MFLAPIClient import MFLAPIClient


class ScoringAndResultsAPIClient(MFLAPIClient):

    @classmethod
    def get_league_standings(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The current league standings for a given league.
        Private league access restricted to league owners.
        """
        # When set to a non-zero value, return a mapping of column keys to column names.
        # This also shows the proper order of the standings columns.
        column_names: str = kwargs.pop("column_names", None)
        filters = [("TYPE", "leagueStandings"), ("L", league_id), ("JSON", 1)]
        cls._add_filter_if_given("COLUMN_NAMES", column_names, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_weekly_results(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The weekly results for a given league/week, including the scores for all starter and non-starter players for all franchises in a league.
        The week parameter can be "YTD" to give all year-to-date weekly results.
        Private league access restricted to league owners.
        """
        # If the week is specified, it returns the data for that week, otherwise the most current data is returned.
        # If the value is 'YTD', then it returns year-to-date data (or the entire season when called on historical leagues).
        week: int = kwargs.pop("week", None)
        filters = [("TYPE", "weeklyResults"), ("L", league_id), ("JSON", 1)]
        cls._add_filter_if_given("W", week, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_live_scoring(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        Live scoring for a given league and week, including:
        - Each franchise's current score
        - How many game seconds remaining that franchise has
        - Players who have yet to play
        - Players who are currently playing
        """
        # If the week is specified, it returns the data for that week, otherwise the most current data is returned.
        week: int = kwargs.pop("week", None)
        # Setting this argument to 1 will return data for non-starters as well
        details: int = kwargs.pop("details", None)
        filters = [("TYPE", "liveScoring"), ("L", league_id), ("JSON", 1)]
        cls._add_filter_if_given("W", week, filters)
        cls._add_filter_if_given("DETAILS", details, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_player_scores(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        All player scores for a given league/week, including all rostered players as well as all free agents.
        Private league access restricted to league owners.
        """
        # If the week is specified, it returns the data for that week, otherwise the current week data is returned.
        # If the value is 'YTD', then it returns year-to-date data.
        # If the value is 'AVG', then it returns a weekly average.
        week: int | str = kwargs.pop("week", None)
        # The year for the data to be returned.
        for_year: int = kwargs.pop("for_year", None)
        # Pass a list of player ids separated by commas (or just a single player id) to receive back just the info on those players.
        players: str = kwargs.pop("players", None)
        # Return only players from this position.
        position: str = kwargs.pop("position", None)
        # If set to 'freeagent', returns only players that are fantasy league free agents.
        status: str = kwargs.pop("status", None)
        # If set, and a league id passed, it re-calculates the fantasy score for each player according to that league's rules.
        # This is only valid when specifying the current year and current week.
        rules: str = kwargs.pop("rules", None)
        # Limit the result to this many players.
        count: int = kwargs.pop("count", None)
        filters = [("TYPE", "playerScores"), ("L", league_id), ("JSON", 1)]
        cls._add_filter_if_given("W", week, filters)
        cls._add_filter_if_given("YEAR", for_year, filters)
        cls._add_filter_if_given("PLAYERS", players, filters)
        cls._add_filter_if_given("POSITION", position, filters)
        cls._add_filter_if_given("STATUS", status, filters)
        cls._add_filter_if_given("RULES", rules, filters)
        cls._add_filter_if_given("COUNT", count, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_projected_scores(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        Given a player ID, calculate the expected fantasy points, using that league's scoring system.
        The system will use the raw stats that fantasysharks.com projects.
        Private league access restricted to league owners.
        """
        # If the week is specified, it returns the projected scores for that week, otherwise the upcoming week is used.
        week: int = kwargs.pop("week", None)
        # Pass a list of player ids separated by commas (or just a single player id) to receive back just the info on those players.
        players: str = kwargs.pop("players", None)
        # Return only players from this position.
        position: str = kwargs.pop("position", None)
        # If set to 'freeagent', returns only players that are fantasy league free agents (note that this refers to players that current free agents, not that were free agents during the specified week).
        status: str = kwargs.pop("status", None)
        # Limit the result to this many players.
        count: int = kwargs.pop("count", None)
        filters = [("TYPE", "projectedScores"), ("L", league_id), ("JSON", 1)]
        cls._add_filter_if_given("W", week, filters)
        cls._add_filter_if_given("PLAYERS", players, filters)
        cls._add_filter_if_given("POSITION", position, filters)
        cls._add_filter_if_given("STATUS", status, filters)
        cls._add_filter_if_given("COUNT", count, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)



from pymfl.api.MFLAPIClient import MFLAPIClient


class CommonLeagueInfoAPIClient(MFLAPIClient):

    @classmethod
    def get_league(cls, *, year: int, league_id: str) -> dict:
        """
        General league setup parameters for a given league.
        Personal user information like name and email addresses only returned to league owners.
        """
        filters = [("TYPE", "league"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_rules(cls, *, year: int, league_id: str) -> dict:
        """
        League scoring rules for a given league.
        """
        filters = ("TYPE", "rules"), ("L", league_id), ("JSON", 1)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_rosters(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The current rosters for all franchises in a league, including player status (active roster, IR, TS), as well as all salary/contract information for that player.
        """
        # When set, the response will include the current roster of just the specified franchise.
        franchise: str = kwargs.pop("franchise", None)
        # If the week is specified, it returns the roster for that week.
        # The week must be less than or equal to the upcoming week.
        # Changes to salary and contract info is not tracked so those fields (if used) always show the current values.
        week: int = kwargs.pop("week", None)
        filters = [("TYPE", "rosters"), ("L", league_id), ("JSON", 1)]
        cls._add_filter_if_given("W", week, filters)
        cls._add_filter_if_given("FRANCHISE", franchise, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_free_agents(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        Fantasy free agents for a given league.
        Private league access restricted to league owners.
        """
        # Return only players from this position.
        position: str = kwargs.pop("position", None)
        filters = [("TYPE", "freeAgents"), ("L", league_id), ("JSON", 1)]
        cls._add_filter_if_given("POSITION", position, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_schedule(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The fantasy schedule for a given league/week.
        Weeks in the past will show the score of each matchup.
        Private league access restricted to league owners.
        """
        # If a week is specified, it returns the fantasy schedule for that week, otherwise the full schedule is returned.
        week: int = kwargs.pop("week", None)
        # If a franchise id is specified, the schedule for just that franchise is returned.
        franchise_id: str = kwargs.pop("franchise_id", None)
        filters = [("TYPE", "schedule"), ("L", league_id), ("JSON", 1)]
        cls._add_filter_if_given("W", week, filters)
        cls._add_filter_if_given("F", franchise_id, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_calendar(cls, *, year: int, league_id: str) -> dict:
        """
        Returns a summary of the league calendar events.
        Access restricted to league owners.
        """
        filters = [("TYPE", "calendar"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_playoff_brackets(cls, *, year: int, league_id: str) -> dict:
        """
        All playoff brackets for a given league.
        Private league access restricted to league owners.
        """
        filters = [("TYPE", "playoffBrackets"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_playoff_bracket(cls, *, year: int, league_id: str, bracket_id: str) -> dict:
        """
        Returns the games (with results if available) of the specified playoff bracket.
        Private league access restricted to league owners.
        """
        filters = [("TYPE", "schedule"), ("L", league_id), ("JSON", 1), ("BRACKET_ID", bracket_id)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)



from .CommonLeagueInfoAPIClient import CommonLeagueInfoAPIClient
from .CommunicationsAPIClient import CommunicationsAPIClient
from .DraftAndAuctionAPIClient import DraftAndAuctionAPIClient
from .FantasyContentAPIClient import FantasyContentAPIClient
from .LeaguePlayersAPIClient import LeaguePlayersAPIClient
from .NFLContentAPIClient import NFLContentAPIClient
from .OtherLeagueInfoAPIClient import OtherLeagueInfoAPIClient
from .ScoringAndResultsAPIClient import ScoringAndResultsAPIClient
from .SessionAPIClient import SessionAPIClient
from .TransactionsAPIClient import TransactionsAPIClient
from .UserFunctionsAPIClient import UserFunctionsAPIClient



from pymfl.api.MFLAPIClient import MFLAPIClient


class LeaguePlayersAPIClient(MFLAPIClient):

    @classmethod
    def get_player_roster_status(cls, *, year: int, league_id: str, player_id_or_ids: str, **kwargs) -> dict:
        """
        Get the player's current roster status.
        The franchise(s) the player is on are listed in the sub-element.
        There may more than one of this for leagues that have multiple copies of players.
        Each of these elements will have a franchise id and status attribute.
        The status attribute can be one of: R (roster), S (starter), NS (non-starter), IR (injured reserve) or TS (taxi squad).
        The R value is only provided when there's no lineup submitted or the caller has no visibility into the lineup.
        If the player is a free agent, there will be a 'is_fa' attribute on the parent element.
        In those cases the elements 'cant_add' and 'locked' attributes may be set indicating whether a player can't be added or is locked.

        player_id_or_ids: Player id or list of player ids separated by commas
        """
        filters = [("TYPE", "playerRosterStatus"), ("L", league_id), ("P", player_id_or_ids), ("JSON", 1)]
        # Week. If a week is specified, it returns the player status for that week.
        # The default is the current Live Scoring week.
        week: int = kwargs.pop("week", None)
        # Franchise ID.
        # If present it uses the franchise id to determine which conference or division to use for the purposes of identifying free agents.
        # If not present it uses the user's franchise id.
        # Only matters on deluxe leagues.
        franchise_id: str = kwargs.pop("franchise_id", None)
        cls._add_filter_if_given("W", week, filters)
        cls._add_filter_if_given("F", franchise_id, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_contest_players(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        On Contest Leagues, this returns all players eligible to be in a franchise's starting lineup.
        While this request can be used by any league, it's best suited for leagues with the loadRosters setting set to either 'contest' or 'setem'.
        Access restricted to league owners.
        """
        filters = [("TYPE", "contestPlayers"), ("L", league_id), ("JSON", 1)]
        # Week. If a week is specified, it returns the players for that week.
        # The default is the upcoming week.
        week: int = kwargs.pop("week", None)
        # Franchise ID.
        # If present it returns the players eligible for that franchise.
        # Only matters when called by the commissioner.
        # When called by an owner, this parameter is ignored and the owner's franchise is used.
        franchise_id: str = kwargs.pop("franchise_id", None)
        cls._add_filter_if_given("W", week, filters)
        cls._add_filter_if_given("F", franchise_id, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_my_watch_list(cls, *, year: int, league_id: str) -> dict:
        """
        My Watch List for the current franchise.
        Access restricted to league owners.
        """
        filters = [("TYPE", "myWatchList"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_salaries(cls, *, year: int, league_id: str) -> dict:
        """
        The current player salaries and contract fields.
        Only players with values are returned.
        If a value is empty it means that the default value is in effect.
        The default values are specified under the player id '0000'.
        """
        filters = [("TYPE", "salaries"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_salary_adjustments(cls, *, year: int, league_id: str) -> dict:
        """
        All extra salary adjustments for a given league.
        Private league access restricted to league owners.
        """
        filters = [("TYPE", "salaryAdjustments"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)



from pymfl.api.MFLAPIClient import MFLAPIClient


class CommunicationsAPIClient(MFLAPIClient):

    @classmethod
    def get_message_board(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        Display a summary of the recent message board posts to a league message board.
        Access restricted to league owners.
        """
        filters = [("TYPE", "messageBoard"), ("L", league_id), ("JSON", 1)]
        # If specified, limit the number of threads to display to this value.
        # Default is 10.
        count: str = kwargs.pop("count", None)
        cls._add_filter_if_given("COUNT", count, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_message_board_thread(cls, *, year: int, league_id: str, thread_id: str) -> dict:
        """
        Display posts in a thread from a league message board.
        Access restricted to league owners.
        """
        filters = [("TYPE", "messageBoardThread"), ("THREAD", thread_id), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_polls(cls, *, year: int, league_id: str) -> dict:
        """
        All current league polls with details as to which polls the current franchise voted on.
        Access restricted to league owners.
        """
        filters = [("TYPE", "polls"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)



from dataclasses import dataclass

from pymfl.exception import MissingYearAPIConfigException


@dataclass(kw_only=True, frozen=True)
class YearAPIConfig:
    """
    Used to hold the API config for a single year/league combination.
    """
    league_id: str
    mfl_user_id: str
    user_agent_name: str
    year: int


class APIConfig:
    """
    This class is used to store and set up initial API configuration.
    It should be set before any API calls are made.
    This is a BORG: https://code.activestate.com/recipes/66531/
    """
    config_by_year_and_league_id: dict[str, YearAPIConfig] = dict()

    @classmethod
    def get_config_by_year_and_league_id(cls, *, year: int, league_id: str) -> YearAPIConfig:
        # TODO: add error handling if year/league combo is not found
        try:
            return cls.config_by_year_and_league_id[f"{year}{league_id}"]
        except KeyError as e:
            raise MissingYearAPIConfigException(
                f"Cannot find YearAPIConfig for year '{year}' and league_id '{league_id}'.")

    @classmethod
    def add_config_for_year_and_league_id(cls, *, year: int, league_id: str, username: str, password: str,
                                          user_agent_name: str):
        from pymfl.api.SessionAPIClient import SessionAPIClient
        mfl_user_id = SessionAPIClient.get_mfl_user_id(year=year, username=username, password=password)
        cls.config_by_year_and_league_id[f"{year}{league_id}"] = YearAPIConfig(year=year,
                                                                               league_id=league_id,
                                                                               mfl_user_id=mfl_user_id,
                                                                               user_agent_name=user_agent_name)



from pymfl.api.MFLAPIClient import MFLAPIClient


class TransactionsAPIClient(MFLAPIClient):

    @classmethod
    def get_transactions(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        All non-pending transactions for a given league.
        Note that this can be a very large set, so it's recommended that you filter the result using one or more of the available parameters.
        If the request comes from an owner in the league, it will return the pending transactions for that owner's franchise.
        If it comes from the commissioner, it will return all pending transactions.
        Private league access restricted to league owners.
        """
        # If the week is specified, it returns the transactions for that week.
        week: int = kwargs.pop("week", None)
        # Returns only transactions of the specified type.
        # Types are: WAIVER, BBID_WAIVER, FREE_AGENT, WAIVER_REQUEST, BBID_WAIVER_REQUEST, TRADE, IR, TAXI, AUCTION_INIT, AUCTION_BID, AUCTION_WON, SURVIVOR_PICK, POOL_PICK.
        # You may also specify a value of * to indicate all transaction types or DEFAULT for the default transaction type set.
        # You can ask for multiple types by separating them with commas.
        trans_type: str = kwargs.pop("trans_type", None)
        # When set, returns just the transactions for the specified franchise.
        franchise: str = kwargs.pop("franchise", None)
        # When set, returns just the transactions for the number of days specified by this parameter.
        days: int = kwargs.pop("days", None)
        # Restricts the results to just this many entries.
        # Note than when this field is specified, only transactions from the most common types are returned.
        count: int = kwargs.pop("count", None)
        filters = [("TYPE", "transactions"), ("L", league_id), ("JSON", 1)]
        cls._add_filter_if_given("W", week, filters)
        cls._add_filter_if_given("TRANS_TYPE", trans_type, filters)
        cls._add_filter_if_given("FRANCHISE", franchise, filters)
        cls._add_filter_if_given("DAYS", days, filters)
        cls._add_filter_if_given("COUNT", count, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_pending_waivers(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        Pending waivers that the current franchise has submitted, but have not yet been processed.
        Access restricted to league owners.
        """
        # When request comes from the league commissioner, this indicates which franchise they want.
        # Pass in '0000' to get trades pending commissioner action).
        franchise_id: str = kwargs.pop("franchise_id", None)
        filters = [("TYPE", "pendingWaivers"), ("L", league_id), ("JSON", 1)]
        cls._add_filter_if_given("FRANCHISE_ID", franchise_id, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_pending_trades(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        Pending trades that the current franchise has offered to other franchises, or has been offered to by other franchises.
        Access restricted to league owners.
        """
        # When request comes from the league commissioner, this indicates which franchise they want.
        # Pass in '0000' to get trades pending commissioner action).
        franchise_id: str = kwargs.pop("franchise_id", None)
        filters = [("TYPE", "pendingTrades"), ("L", league_id), ("JSON", 1)]
        cls._add_filter_if_given("FRANCHISE_ID", franchise_id, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_trade_bait(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The Trade Bait for all franchises in a league.
        Private league access restricted to league owners.
        """
        # When set, this will also return draft picks offered.
        # Current year draft picks look like DP_02_05 which refers to the 3rd round 6th pick
        # (the round and pick values in the string are one less than the actual round/pick).
        # For future years picks, they are identified like FP_0005_2018_2
        # where 0005 refers to the franchise id who originally owns the draft pick, then the year and then the round
        # (in this case the rounds are the actual rounds, not one less).
        # This also includes Blind Bid dollars (in leagues that use them), which will be specified as BB_10 to indicate $10 in blind bid dollars.
        include_draft_picks: bool = kwargs.pop("include_draft_picks", None)
        filters = [("TYPE", "tradeBait"), ("L", league_id), ("JSON", 1)]
        cls._add_filter_if_given("INCLUDE_DRAFT_PICKS", include_draft_picks, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_assets(cls, *, year: int, league_id: str) -> dict:
        """
        All tradable assets (players, current year draft picks, future draft picks) for a given league.
        Access restricted to league owners.
        """
        filters = [("TYPE", "assets"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)



from pymfl.api.MFLAPIClient import MFLAPIClient


class NFLContentAPIClient(MFLAPIClient):

    @classmethod
    def get_injuries(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The player ID, status (IR, Out, Doubtful, Questionable, Inactive, etc.) and details (i.e., 'Knee', 'Foot', 'Ribs', etc.)
        of all players on the NFL injury report.
        The report data is updated daily during the season and pre-season.
        The timestamp attribute tells you the last time this data was updated.
        """
        filters = [("TYPE", "injuries"), ("JSON", 1)]
        # If the week is not specified, it defaults to the most recent week that injury data is available.
        week: int = kwargs.pop("week", None)
        cls._add_filter_if_given("W", week, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_nfl_schedule(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The NFL schedule for one week of the season, including:
            - The scheduled kickoff of the game
            - Home team (and their score)
            - Away team (and their score)
        Starting with the 2020 season, this request will no longer update while games are in progress, and it's meant to be used to get the NFL schedule.
        To obtain the current scores of games, please check out this FAQ.
        """
        filters = [("TYPE", "nflSchedule"), ("JSON", 1)]
        # If the week is not specified, it defaults to the current week.
        # If set to 'ALL', it returns the full season schedule.
        week: str = kwargs.pop("week", None)
        cls._add_filter_if_given("W", week, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_nfl_bye_weeks(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        The bye weeks for every NFL team.
        """
        filters = [("TYPE", "nflByeWeeks"), ("JSON", 1)]
        # If the week is specified, it returns just the teams with a bye in that week.
        week: int = kwargs.pop("week", None)
        cls._add_filter_if_given("W", week, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_points_allowed(cls, *, year: int, league_id: str) -> dict:
        """
        Fantasy points allowed by each NFL team, broken out by position.
        """
        filters = [("TYPE", "pointsAllowed"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)



from pymfl.api.MFLAPIClient import MFLAPIClient


class DraftAndAuctionAPIClient(MFLAPIClient):

    @classmethod
    def get_draft_results(cls, *, year: int, league_id: str) -> dict:
        """
        Draft results for a given league.
        Note that this data may be up to 15 minutes delayed as it is meant to display draft results after a draft is completed.
        To access this data while drafts are in progress, check out this FAQ: https://api.myfantasyleague.com/2022/support?FAQ=935
        Private league access restricted to league owners.
        """
        filters = [("TYPE", "draftResults"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_auction_results(cls, *, year: int, league_id: str) -> dict:
        """
        Auction results for a given league.
        Private league access restricted to league owners.
        """
        filters = [("TYPE", "auctionResults"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_my_draft_list(cls, *, year: int, league_id: str) -> dict:
        """
        My Draft List for the current franchise.
        Access restricted to league owners.
        """
        filters = [("TYPE", "myDraftList"), ("L", league_id), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)



import setuptools

pkg_vars = dict()
with open("pymfl/_version.py") as f:
    exec(f.read(), pkg_vars)

with open("README.md") as f:
    read_me = f.read()

with open("LICENSE") as f:
    license_ = f.read()

setuptools.setup(
    name="pymfl",
    version=pkg_vars["__version__"],
    author="Joey Greco",
    author_email="joeyagreco@gmail.com",
    description="Python wrapper for the myfantasyleague API.",
    long_description_content_type="text/markdown",
    long_description=read_me,
    license=license_,
    include_package_data=True,
    packages=setuptools.find_packages(exclude=("test", "docs")),
    install_requires=["setuptools"]
)



from pymfl.api.MFLAPIClient import MFLAPIClient


class UserFunctionsAPIClient(MFLAPIClient):

    @classmethod
    def get_my_leagues(cls, *, year: int, league_id: str, **kwargs) -> dict:
        """
        All leagues of the current user.
        Private league access restricted to league owners.
        Personal user information, like name and email addresses only returned to league owners.
        """
        filters = [("TYPE", "myleagues"), ("JSON", 1)]
        franchise_names: str = kwargs.pop("franchise_names", None)
        cls._add_filter_if_given("FRANCHISE_NAMES", franchise_names, filters)
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)

    @classmethod
    def get_league_search_results(cls, *, year: int, league_id: str, search: str) -> dict:
        """
        Returns all leagues on MyFantasyLeague.com whose name match the specified string.
        """
        filters = [("TYPE", "leagueSearch"), ("SEARCH", search), ("JSON", 1)]
        url = cls._build_route(cls._MFL_APP_BASE_URL, year, cls._EXPORT_ROUTE)
        url = cls._add_filters(url, *filters)
        return cls._get_for_year_and_league_id(url=url, year=year, league_id=league_id)



import configparser
import os


class ConfigReader:
    """
    Used to read from .properties files
    """
    __propertiesFileName = "app.properties"

    @classmethod
    def get(cls, section: str, name: str, asType=None) -> str:
        configParser = configparser.ConfigParser(converters={'list': lambda x: [i.strip() for i in x.split(',')]})
        propertiesFilePath = os.path.abspath(
            os.path.join(os.path.dirname(os.path.realpath(__file__)), f"../{cls.__propertiesFileName}"))
        configParser.read(propertiesFilePath)
        if asType == list:
            return configParser.getlist(section, name)
        elif asType is None:
            return configParser[section][name]
        else:
            raise ValueError(f"Type conversion for '{asType}' not supported.")
